'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

/** High Order Component which
 * @param {Function} mapArgsToProps -
 * @param {Function} generateComponentKey -
 * @param {Function} mapDataToProps -
 * @param {Function} getInitialData -
 */
var withGetInitialData$1 = function withGetInitialData(_ref) {
  var mapArgsToProps = _ref.mapArgsToProps,
      generateComponentKey = _ref.generateComponentKey,
      mapDataToProps = _ref.mapDataToProps,
      _getInitialData = _ref.getInitialData;
  return function (Component) {
    var GetInitialData = function (_React$Component) {
      inherits(GetInitialData, _React$Component);

      function GetInitialData(props) {
        classCallCheck(this, GetInitialData);

        var _this = possibleConstructorReturn(this, (GetInitialData.__proto__ || Object.getPrototypeOf(GetInitialData)).call(this, props));

        var componentKey = _this.getComponentKey(generateComponentKey(Component, props));
        _this.state = {
          data: (props.initialData || {})[componentKey],
          isLoading: false
        };
        return _this;
      }

      /** Static method called during SSR to fetch data
       * @param {Function(componentKey, data} setData - A function used to save the fetched data while SSR.
       * @param {Object} args - all arguments applied
       * @return {Promise} - Promise that the data will be fetched
       */


      createClass(GetInitialData, [{
        key: 'componentDidMount',


        /** React `componentDidMount` phase.
         * As far as data fetching, it is only called on the client side when the component is mounted.
         * A double fetch has to be prevented (fetch data when the server has already provided it)
         */
        value: function componentDidMount() {
          this.fetchDataIfNeeded();
        }

        /** React `componentWillReceiveProps` phase.
         * It is called when props have changed.
         * We'll have to check if the change involves a new data fetch.
         */

      }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
          // If Component didn't implement, do nothing
          if (!_getInitialData) return;

          var key = this.getComponentKey();
          var nextKey = generateComponentKey(Component, nextProps);

          if (key === nextKey) return;
          this.dismissLoadedComponent();
          this.fetchDataIfNeeded(nextProps, nextKey);
        }
      }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
          this.dismissLoadedComponent();
        }
      }, {
        key: 'render',
        value: function render() {
          var _state = this.state,
              isLoading = _state.isLoading,
              data = _state.data;

          var _props2 = this.props,
              initialData = _props2.initialData,
              hasLoadedComponent = _props2.hasLoadedComponent,
              dismissLoadedComponent = _props2.dismissLoadedComponent,
              _props = objectWithoutProperties(_props2, ['initialData', 'hasLoadedComponent', 'dismissLoadedComponent']);

          var props = _extends({}, _props, mapDataToProps(data || {}));

          return React.createElement(Component, _extends({ isLoading: isLoading }, props));
        }

        /** Check if data has to be fetched, if it does, it calls `getInitialData` of Component.
         * The idea of this method is to prevent the double fetch when hydrating SSR code.
         * It checks if the Component's key has been fetched.
         */

      }, {
        key: 'fetchDataIfNeeded',
        value: function fetchDataIfNeeded(nextProps, nextKey) {
          var _this2 = this;

          // If Component didn't implement, do nothing
          if (!_getInitialData) return;

          var key = this.getComponentKey(nextKey);
          if (this.props.hasLoadedComponent(key)) return;

          _getInitialData(nextProps || this.props, {
            setLoading: function setLoading(b) {
              return _this2.setState({ isLoading: b });
            },
            setData: function setData(d) {
              return _this2.setState({ isLoading: false, data: d });
            }
          });
        }

        /**
         */

      }, {
        key: 'dismissLoadedComponent',
        value: function dismissLoadedComponent() {
          // If Component didn't implement, do nothing
          if (!_getInitialData) return;

          var key = this.getComponentKey();
          this.props.dismissLoadedComponent(key);
        }
      }, {
        key: 'getComponentKey',
        value: function getComponentKey(nextKey) {
          if (nextKey) this.key = nextKey;

          if (!this.key) this.key = generateComponentKey(Component, this.props);

          return this.key;
        }
      }], [{
        key: 'getInitialData',
        value: function getInitialData(_setData) {
          // If Component didn't implement `getInitialData` return a promise
          if (!_getInitialData) return Promise.resolve(null);

          for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          var props = mapArgsToProps(args);
          var componentKey = generateComponentKey(Component, props);

          return _getInitialData(props, {
            // Ignore setLoading as the server will render after
            setLoading: function setLoading() {},
            // Use setData in order to retrieve data
            setData: function setData(data) {
              return _setData(componentKey, data);
            }
          });
        }
      }]);
      return GetInitialData;
    }(React.Component);

    GetInitialData.propTypes = {
      initialData: PropTypes.object,
      hasLoadedComponent: PropTypes.func, //.isRequired,
      dismissLoadedComponent: PropTypes.func //.isRequired,
    };

    return GetInitialData;
  };
};

module.exports = withGetInitialData$1;
//# sourceMappingURL=react-data-ssr.js.map
